
/* Name:        Serial to Stepper Control
*  Description: An attempt to have full speed stepper control while receiving data from serial port.
*  Created by:  Corey Strachn aka pRofiT
*  Date:        06/17/2019
*  Platform:    Arduino Mega 2560, RAMPS 1.4
*  Arduino Ver: 1.6.5
*/


#include <AccelStepper.h> // for smooth stepper control


unsigned long lastReceiveTime = 0;
unsigned long currentTime = 0;

//RAMPS 1.4 constants

#define X_STEP_PIN         54
#define X_DIR_PIN          55
#define X_ENABLE_PIN       38 // need to incorperate enable pin somehow
#define X_MIN_PIN           3 // limits not needed
#define X_MAX_PIN           2 // limits not needed

#define Y_STEP_PIN         60
#define Y_DIR_PIN          61
#define Y_ENABLE_PIN       56
#define Y_MIN_PIN          14
#define Y_MAX_PIN          15

#define Z_STEP_PIN         46
#define Z_DIR_PIN          48
#define Z_ENABLE_PIN       62
#define Z_MIN_PIN          18
#define Z_MAX_PIN          19

#define E_STEP_PIN         26
#define E_DIR_PIN          28
#define E_ENABLE_PIN       24

#define Q_STEP_PIN         36
#define Q_DIR_PIN          34
#define Q_ENABLE_PIN       30

#define SDPOWER            -1
#define SDSS               53
#define LED_PIN            13

#define FAN_PIN            9

#define PS_ON_PIN          12
#define KILL_PIN           -1

#define HEATER_0_PIN       10
#define HEATER_1_PIN       8
#define TEMP_0_PIN          13   // ANALOG NUMBERING
#define TEMP_1_PIN          14   // ANALOG NUMBERING

// Global vars. i know but you can't return char arrays in functions. without extra work. and it ends up almost same as global
// static probably not needed now that they are global?
const byte numChars = 32;
static bool dataReady; // bool to determine if data is ready to read.
static char data[numChars]; // data received
static int dataPos; // keep track of current data position
static bool recieveInProgress;

// Define the stepper motors and the pins we will use

AccelStepper LeftWheel(1, X_STEP_PIN, X_DIR_PIN);   // X Driver Left Wheel
AccelStepper RightWheel(1, Y_STEP_PIN, Y_DIR_PIN);  // Y Driver Right Wheel

int leftWheelSpeed = 0; // start at 0 speed
int rightWheelSpeed = 0; // start at 0 speed

int prevLeftWheelSpeed = 0; // start at 0 speed
int prevRightWheelSpeed = 0; // start at 0 speed



// fyi 0.00027777 for once per hour, Max about 1000 steps a second
const int minStepperSpeed = -1000; // not sure what this will be but lets get some constants going.
const int maxStepperSpeed = 1000;

void setup()
{
Serial.begin(115200);
while (!Serial) { } // wait for serial to be ready. Some arduinos need this

pinMode(X_ENABLE_PIN, OUTPUT); 
pinMode(Y_ENABLE_PIN, OUTPUT); 
pinMode(X_STEP_PIN, OUTPUT); 
pinMode(X_DIR_PIN, OUTPUT);
pinMode(Y_STEP_PIN, OUTPUT); 
pinMode(Y_DIR_PIN, OUTPUT); 
digitalWrite(X_ENABLE_PIN , LOW);
digitalWrite(Y_ENABLE_PIN , LOW);

LeftWheel.setMaxSpeed(maxStepperSpeed);
RightWheel.setMaxSpeed(maxStepperSpeed);

resetData(); // setup recieve data
 
}


// Main Loop
void loop()
{
  currentTime = millis();
  if(currentTime - lastReceiveTime > 1000 )
    { // If current time is more then 1 second since we have recived the last data, that means we have lost connection (means we need to send data every second or faster to keep moving.
    resetData(); // If connection is lost, reset the data. It prevents unwanted behavior, for example if a drone has a throttle up, if we lose connection it can keep flying away if we dont reset the function
    }
    
  recieveSerialData(); // get data one byte at a loop
  if(dataReady) checkCommands(); // if data ready then read command
  runSteppers(); // run stepp controls


}


// This function will read one byte at a time each call if available and set bools when data is ready
void recieveSerialData()
{
  // example data <x,y> <100,100> forward <-100,100> spin left <100,-100> spin right <-100,-100> reverse etc.. i think i sbetter then command and speed. unless special commands can be sent like shake or 180 spin attack etc...?
 
  const byte startbyte = '<'; // for begining of data
  const byte endbyte = '>'; // for end of data
  byte inByte;
 
  if(dataReady) return; // exit if data is ready to be read. dont waste time in here. Needs to be read before more can come in.
 
  if (Serial.available() > 0)
    {
    lastReceiveTime = millis(); 
    inByte = Serial.read(); // get a byte of data
    if(recieveInProgress & inByte!=endbyte)
      {
        data[dataPos] = inByte;
        dataPos++; // increase byte count at the end
      }
    if(inByte == startbyte)
      {
        resetData();
        recieveInProgress = true;
      }
     if(inByte == endbyte)
      {
        data[dataPos] = char(0);// end byte
        recieveInProgress = false;
        dataReady = true;
      }  
    }
}


void resetData()
{
  dataReady = false; // clear data ready so more info can be read
  data[0] = char(0);
  dataPos = 0;
  recieveInProgress = false;
}

 

void checkCommands()
{
  char * leftSpeedStr;
  char * rightSpeedStr;
  int leftSpeedInt;
  int rightSpeedInt;

  // get command and speed
  leftSpeedStr = strtok(data,","); // get data before the comma "100,100"
  rightSpeedStr = strtok(NULL,","); // get data after the comma
  // convert string to int
  leftSpeedInt = atoi(leftSpeedStr); 
  rightSpeedInt = atoi(rightSpeedStr);

  // limit the speed
  leftSpeedInt = constrain(leftSpeedInt, minStepperSpeed, maxStepperSpeed); 
  rightSpeedInt = constrain(rightSpeedInt, minStepperSpeed, maxStepperSpeed);

  // get previous values
  prevLeftWheelSpeed = leftWheelSpeed; 
  prevRightWheelSpeed = rightWheelSpeed;

  // set globals speed
  leftWheelSpeed = leftSpeedInt; 
  rightWheelSpeed = rightSpeedInt;

  // we reda the data and now we can clear settings so serial can recieve next data packet.
  resetData();
}

 

void runSteppers()
{
  if(leftWheelSpeed != prevLeftWheelSpeed)
    {
      LeftWheel.setSpeed(leftWheelSpeed);
      Serial.println(leftWheelSpeed);
      prevLeftWheelSpeed = leftWheelSpeed;
    }  
    
  if(rightWheelSpeed != prevRightWheelSpeed)
    {
      RightWheel.setSpeed(rightWheelSpeed);
      Serial.println(rightWheelSpeed);
      prevRightWheelSpeed = rightWheelSpeed;
    }
  
  LeftWheel.runSpeed(); // runSpeed the stepper motors. each loop. runspeed will run at the speed set.
  RightWheel.runSpeed(); // runSpeed the stepper motors. each loop
}
